/**
 * @file Loader全局方法
 */

var fs = require('fs');
/**
 * 初始化方法
 *
 * @param {string} keypath 模块命名时需要去除的路径
 * @param {string} path 要加载的路径
 * @param {string} context 声明上下文
 * @param {string} type 异步加载or同步加载
 */
module.exports = function (keypath, path, context,type) {
    context = context || global;
    var files = fs.readdirSync(bun.ROOT_PATH + path);
    files.forEach(function (filename) {
        var self = __filename.substring(path.length + 1);
        if (filename === self) {
            return;
        }
        var stat = fs.lstatSync(bun.ROOT_PATH + path + '/' + filename);
        if (stat.isDirectory()) {
            bun.Loader(keypath, path + '/' + filename,context,type);
            return;
        }
        var pos = filename.lastIndexOf('.');

        if (pos === -1) {
            return;
        }

        var filePrefix = filename.substr(0, pos);
        var filePostfix = filename.substr(pos + 1);
        if (filePrefix.length < 1 || filePostfix.length < 1 || filePostfix !== 'js') {
            return;
        }

        // 拼接方法名
        var newpath = path;
        if ('/'+keypath === newpath) {
            newpath = '';
        } else {
            newpath = path.replace('/'+keypath+'/','');
        }
        
        patharr = newpath.split('/');
        patharr.push(filePrefix);
        var arr = [];
        for (var i = 0; i < patharr.length; i++) {
            if (!patharr[i]) {
                continue;
            }
            // 首字母大写
            arr.push(patharr[i].replace(/^\S/g,function(s){return s.toUpperCase();}));
        }
        
        var key = arr.join('_');

        // 同步加载策略
        if (type === 'async') {
            Object.defineProperty(context, key, {
              get : function(){
                var Model = require(bun.ROOT_PATH + path + '/' + filePrefix);
                return Model;
              },
              enumerable : true,
              configurable : false
            });
            return;
        }
        var Model = require(bun.ROOT_PATH + path + '/' + filePrefix);

        if (typeof Model === 'function') {
            
            if (context[key]) {
                throw new Error('Repeated method name: ' + key + ' in file: ' + filename);
            }
            context[key] = (function () {
                return Model;
            })();

        }

    });
}
